"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var LegacyNetworkMember_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.LegacyNetworkMember = void 0;
const tools_common_1 = require("@affinidi/tools-common");
const shared_dto_1 = require("../dto/shared.dto");
const ParametersValidator_1 = require("../shared/ParametersValidator");
const getOptionsFromEnvironment_1 = require("../shared/getOptionsFromEnvironment");
const WalletStorageService_1 = __importDefault(require("../services/WalletStorageService"));
const PhoneIssuerService_1 = require("../services/PhoneIssuerService");
const EmailIssuerService_1 = require("../services/EmailIssuerService");
const BaseNetworkMember_1 = require("./BaseNetworkMember");
const Util_1 = require("./Util");
/**
 * @deprecated, will be removed in SDK v7
 */
let LegacyNetworkMember = LegacyNetworkMember_1 = class LegacyNetworkMember extends BaseNetworkMember_1.BaseNetworkMember {
    constructor(userData, dependencies, options) {
        super(userData, dependencies, options);
        const { basicOptions: { phoneIssuerBasePath, emailIssuerBasePath }, } = this._options;
        this._phoneIssuer = new PhoneIssuerService_1.PhoneIssuerService({ basePath: phoneIssuerBasePath });
        this._emailIssuer = new EmailIssuerService_1.EmailIssuerService({ basePath: emailIssuerBasePath });
    }
    /**
     * @description Initiates the phone number verification flow
     * @deprecated
     * @param config - Configuration options
     * @param config.apiKey - They api access key to the issuer service
     * @param config.phoneNumber - The phone number to send the confirmation code to
     * @param config.isWhatsAppNumber - Whether the phone number is a WhatsApp number
     * @param config.id - The id of the request, this is for the caller to be able to identify the credential in the verify step
     * @param config.holder - The DID of the user who will recieve the VC (owner of the phone number)
     * @returns intitiate response data, including the status of the request
     */
    async initiatePhoneCredential({ apiKey, phoneNumber, isWhatsAppNumber, id, holder, }) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: apiKey },
            { isArray: false, type: 'string', isRequired: true, value: phoneNumber },
            { isArray: false, type: 'boolean', isRequired: false, value: isWhatsAppNumber },
            { isArray: false, type: 'string', isRequired: true, value: id },
            { isArray: false, type: 'string', isRequired: true, value: holder },
        ]);
        return this._phoneIssuer.initiate({ apiKey, phoneNumber, isWhatsAppNumber, id, holder });
    }
    /**
     * @description Finishes the phone number verification flow
     * @deprecated
     * @param config - Configuration options
     * @param config.apiKey - They api access key to the issuer service
     * @param config.code - The code the user recieved
     * @param config.id - The id of the request, must match the ID given in the initiate step
     * @param config.holder - The DID of the user who will recieve the VC (owner of the phone number)
     * @returns verify response data, including the issued VC(s)
     */
    async verifyPhoneCredential({ apiKey, code, id, holder, }) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: apiKey },
            { isArray: false, type: 'string', isRequired: true, value: code },
            { isArray: false, type: 'string', isRequired: true, value: id },
            { isArray: false, type: 'string', isRequired: true, value: holder },
        ]);
        return this._phoneIssuer.verify({ apiKey, code, id, holder });
    }
    /**
     * @description Initiates the email address verification flow
     * @deprecated
     * @param config - Configuration options
     * @param config.apiKey - They api access key to the issuer service
     * @param config.emailAddress - The email address to send the confirmation code to
     * @param config.id - The id of the request, this is for the caller to be able to identify the credential in the verify step
     * @param config.holder - The DID of the user who will recieve the VC (owner of the email address)
     * @returns intitiate response data, including the status of the request
     */
    async initiateEmailCredential({ apiKey, emailAddress, id, holder, }) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: apiKey },
            { isArray: false, type: 'string', isRequired: true, value: emailAddress },
            { isArray: false, type: 'string', isRequired: true, value: id },
            { isArray: false, type: 'string', isRequired: true, value: holder },
        ]);
        return this._emailIssuer.initiate({ apiKey, emailAddress, id, holder });
    }
    /**
     * @description Finishes the email address verification flow
     * @deprecated
     * @param config - Configuration options
     * @param config.apiKey - They api access key to the issuer service
     * @param config.code - The code the user recieved
     * @param config.id - The id of the request, must match the ID given in the initiate step
     * @param config.holder - The DID of the user who will recieve the VC (owner of the email address)
     * @returns verify response data, including the issued VC(s)
     */
    async verifyEmailCredential({ apiKey, code, id, holder, }) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: apiKey },
            { isArray: false, type: 'string', isRequired: true, value: code },
            { isArray: false, type: 'string', isRequired: true, value: id },
            { isArray: false, type: 'string', isRequired: true, value: holder },
        ]);
        return this._emailIssuer.verify({ apiKey, code, id, holder });
    }
    getPublicKeyHexFromDidDocument(didDocument) {
        return Util_1.Util.getPublicKeyHexFromDidDocument(didDocument);
    }
    /**
     * @description Creates DID and anchors it
     * 1. generate seed/keys
     * 2. build DID document
     * 3. sign DID document
     * 4. store DID document in IPFS
     * 5. anchor DID with DID document ID from IPFS
     * @param password - encryption key which will be used to encrypt randomly created seed/keys pair
     * @param inputOptions - optional parameter { registryUrl: 'https://affinity-registry.apse1.dev.affinidi.io' }
     * @returns
     *
     * did - hash from public key (your decentralized ID)
     *
     * encryptedSeed - seed is encrypted by provided password. Seed - it's a source to derive your keys
     */
    static async register(dependencies, inputOptions, password) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: password },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        return LegacyNetworkMember_1._register(dependencies, options, password);
    }
    static async anchorDid(encryptedSeed, password, didDocument, nonce, inputOptions) {
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        return LegacyNetworkMember_1._anchorDid(encryptedSeed, password, didDocument, nonce, options);
    }
    /**
     * @description Retrieves a VC based on signup information
     * @param idToken - idToken received from cognito
     * @returns an object with a flag, identifying whether new account was created, and initialized instance of SDK
     */
    async getSignupCredentials(idToken, inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: idToken },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        return this._getSignupCredentials(idToken, options);
    }
    /**
     * @description Retrieves a VC based on signup information
     * @param idToken - idToken received from cognito
     * @returns an object with a flag, identifying whether new account was created, and initialized instance of SDK
     */
    async _getSignupCredentials(idToken, { basicOptions: { env, keyStorageUrl, issuerUrl }, accessApiKey }) {
        const credentialOfferToken = await WalletStorageService_1.default.getCredentialOffer(idToken, keyStorageUrl, {
            env,
            accessApiKey,
        });
        const credentialOfferResponseToken = await this.createCredentialOfferResponseToken(credentialOfferToken);
        return WalletStorageService_1.default.getSignedCredentials(idToken, credentialOfferResponseToken, {
            accessApiKey,
            issuerUrl,
            keyStorageUrl,
        });
    }
};
LegacyNetworkMember = LegacyNetworkMember_1 = __decorate([
    (0, tools_common_1.profile)()
], LegacyNetworkMember);
exports.LegacyNetworkMember = LegacyNetworkMember;
//# sourceMappingURL=LegacyNetworkMember.js.map