"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var LegacyNetworkMemberWithFactories_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.LegacyNetworkMemberWithFactories = void 0;
const common_1 = require("@affinidi/common");
const tools_common_1 = require("@affinidi/tools-common");
const WalletStorageService_1 = __importDefault(require("../services/WalletStorageService"));
const shared_dto_1 = require("../dto/shared.dto");
const getDidData_1 = require("../shared/getDidData");
const getOptionsFromEnvironment_1 = require("../shared/getOptionsFromEnvironment");
const ParametersValidator_1 = require("../shared/ParametersValidator");
const randomBytes_1 = require("../shared/randomBytes");
const user_management_1 = require("@affinidi/user-management");
const BaseNetworkMember_1 = require("./BaseNetworkMember");
const LegacyNetworkMember_1 = require("./LegacyNetworkMember");
const createUserManagementService_1 = require("../shared/createUserManagementService");
const normalizeShortPassword_1 = require("../shared/normalizeShortPassword");
const generatePassword_1 = require("../shared/generatePassword");
/**
 * @deprecated, will be removed in SDK v7
 */
let LegacyNetworkMemberWithFactories = LegacyNetworkMemberWithFactories_1 = class LegacyNetworkMemberWithFactories extends LegacyNetworkMember_1.LegacyNetworkMember {
    constructor(userData, dependencies, options) {
        super(userData, dependencies, options);
        this._keysService = new common_1.KeysService(userData.encryptedSeed, userData.password);
        this._userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        this.cognitoUserTokens = userData.cognitoUserTokens;
    }
    /**
     * @description Returns user's accessToken
     * @returns encrypted seed
     */
    get accessToken() {
        return this.cognitoUserTokens.accessToken;
    }
    /**
     * @description Checks if registration for the user was completed
     * @param username - a valid email, phone number or arbitrary username
     * @param inputOptions - object with environment, dev is default { env: 'dev' }
     * @returns `true` if user is uncofirmed in Cognito, and `false` otherwise.
     */
    static async isUserUnconfirmed(username, inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: username },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        return userManagementService.doesUnconfirmedUserExist(username);
    }
    /**
     * @description Initilizes instance of SDK from seed
     * @param seedHexWithMethod - seed for derive keys in string hex format
     * @param inputOptions - optional parameter { registryUrl: 'https://affinity-registry.apse1.dev.affinidi.io' }
     * @param password - optional password, will be generated, if not provided
     * @returns initialized instance of SDK
     */
    static async fromSeed(dependencies, seedHexWithMethod, inputOptions, password = null) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: seedHexWithMethod },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: false, value: password },
        ]);
        let passwordBuffer;
        if (password) {
            passwordBuffer = common_1.KeysService.normalizePassword(password);
        }
        else {
            passwordBuffer = await (0, randomBytes_1.randomBytes)(32);
            password = passwordBuffer.toString('hex');
        }
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userData = (0, getDidData_1.withDidData)({
            encryptedSeed: await common_1.KeysService.encryptSeed(seedHexWithMethod, passwordBuffer),
            password,
        });
        return new LegacyNetworkMemberWithFactories_1({ ...userData, cognitoUserTokens: undefined }, dependencies, options);
    }
    /**
     * @description Pulls encrypted seed from Affinity Guardian Wallet
     * 1. get AWS cognito access token
     * 2. using access token pull encrypted seed from Affinity Guardian Wallet
     * associated with this AWS Cognito userId
     * @param username - email or phone number used to create Affinity user
     * @param inputPassword - password used on create Affinity user
     */
    async pullEncryptedSeed(username, inputPassword) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: username },
            { isArray: false, type: 'string', isRequired: true, value: inputPassword },
        ]);
        const password = (0, normalizeShortPassword_1.normalizeShortPassword)(inputPassword, username);
        const { accessToken } = await this._userManagementService.logInWithPassword(username, password);
        const { encryptedSeed } = await this._keyManagementService.pullKeyAndSeed(accessToken);
        return encryptedSeed;
    }
    /**
     * @description Saves encrypted seed to Guardian Wallet
     * @param username - email/phoneNumber, registered in Cognito
     * @param inputPassword - password for Cognito user
     * @param token - Cognito access token, required for authorization.
     * If not provided, in order to get it, sign in to Cognito will be initiated.
     */
    async storeEncryptedSeed(username, inputPassword, token = undefined) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: username },
            { isArray: false, type: 'string', isRequired: true, value: inputPassword },
            { isArray: false, type: 'jwt', isRequired: false, value: token },
        ]);
        let accessToken = token;
        /* istanbul ignore else: code simplicity */
        if (!token) {
            const password = (0, normalizeShortPassword_1.normalizeShortPassword)(inputPassword, username);
            this.cognitoUserTokens = await this._userManagementService.logInWithPassword(username, password);
            accessToken = this.cognitoUserTokens.accessToken;
        }
        const { seedHexWithMethod } = this._keysService.decryptSeed();
        await this._keyManagementService.pullEncryptionKeyAndStoreEncryptedSeed(accessToken, seedHexWithMethod);
    }
    /**
     * @description Initiates passwordless login to Affinity
     * @param username - email/phoneNumber, registered in Cognito
     * @param options - optional parameters with specified environment
     * @param messageParameters - optional parameters with specified welcome message
     * @returns token
     */
    static async passwordlessLogin(login, inputOptions, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: login },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        return await userManagementService.initiateLogInPasswordless(login, messageParameters);
    }
    /**
     * @description Completes login
     * @param token - received from #passwordlessLogin method
     * @param confirmationCode - OTP sent by AWS Cognito/SES
     * @param options - optional parameters for BaseNetworkMember initialization
     * @returns initialized instance of SDK
     */
    static async completeLoginChallenge(dependencies, token, confirmationCode, inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: token },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const keyManagementService = (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const cognitoUserTokens = await userManagementService.completeLogInPasswordless(token, confirmationCode);
        const userData = await keyManagementService.pullUserData(cognitoUserTokens.accessToken);
        return new LegacyNetworkMemberWithFactories_1({ ...userData, cognitoUserTokens }, dependencies, options);
    }
    /**
     * @description Signs out current user
     */
    async signOut(inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([{ isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions }]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const newTokens = await userManagementService.logOut(this.cognitoUserTokens);
        this.cognitoUserTokens = newTokens;
    }
    /**
     * @description Initiates reset password flow
     * @param username - email/phoneNumber, registered in Cognito
     * @param options - optional parameters with specified environment
     * @param messageParameters - optional parameters with specified welcome message
     */
    static async forgotPassword(login, inputOptions, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: login },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        await userManagementService.initiateForgotPassword(login, messageParameters);
    }
    /**
     * @description Completes reset password flow
     * @param username - email/phoneNumber, registered in Cognito
     * @param confirmationCode - OTP sent by AWS Cognito/SES
     * @param newPassword - new password
     * @param options - optional parameters with specified environment
     */
    static async forgotPasswordSubmit(login, confirmationCode, newPassword, inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: login },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
            { isArray: false, type: 'string', isRequired: true, value: newPassword },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        await userManagementService.completeForgotPassword(login, confirmationCode, newPassword);
    }
    /**
     * @description Logins to Affinity with login and password
     * @param username - email/phoneNumber, registered in Cognito
     * @param inputPassword - password for Cognito user
     * @param options - optional parameters for BaseNetworkMember initialization
     * @returns initialized instance of SDK
     */
    static async fromLoginAndPassword(dependencies, username, inputPassword, inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: username },
            { isArray: false, type: 'password', isRequired: true, value: inputPassword },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const keyManagementService = (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const password = (0, normalizeShortPassword_1.normalizeShortPassword)(inputPassword, username);
        const cognitoUserTokens = await userManagementService.logInWithPassword(username, password);
        const userData = await keyManagementService.pullUserData(cognitoUserTokens.accessToken);
        return new LegacyNetworkMemberWithFactories_1({ ...userData, cognitoUserTokens }, dependencies, options);
    }
    /**
     * @description Initiates sign up flow to Affinity wallet with already created did
     * @param keyParams - { ecnryptedSeed, password } - previously created keys to be storead at wallet.
     * @param username - arbitrary username, email or phoneNumber
     * @param inputPassword - is required if arbitrary username was provided.
     * It is optional and random one will be generated, if not provided when
     * email or phoneNumber was given as a username.
     * @param options - optional parameters with specified environment
     * @param messageParameters - optional parameters with specified welcome message
     * @returns token or, in case when arbitrary username was used, it returns
     * initialized instance of SDK
     */
    static async signUpWithExistsEntity(dependencies, keyParams, login, inputPassword, inputOptions, messageParameters) {
        const { isUsername } = (0, user_management_1.validateUsername)(login);
        if (!isUsername) {
            return LegacyNetworkMemberWithFactories_1._signUpByEmailOrPhone(login, inputPassword, inputOptions, messageParameters);
        }
        const username = login;
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.KeyParams, isRequired: true, value: keyParams },
            { isArray: false, type: 'string', isRequired: true, value: username },
            { isArray: false, type: 'password', isRequired: true, value: inputPassword },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        LegacyNetworkMemberWithFactories_1._validateKeys(keyParams);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const password = (0, normalizeShortPassword_1.normalizeShortPassword)(inputPassword, username);
        const cognitoTokens = await userManagementService.signUpWithUsernameAndConfirm(username, password);
        return LegacyNetworkMemberWithFactories_1._confirmSignUp(dependencies, options, cognitoTokens, password, keyParams);
    }
    static async _signUpByUsernameAutoConfirm(dependencies, username, inputPassword, inputOptions, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: username },
            { isArray: false, type: 'password', isRequired: true, value: inputPassword },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const password = (0, normalizeShortPassword_1.normalizeShortPassword)(inputPassword, username);
        const cognitoTokens = await userManagementService.signUpWithUsernameAndConfirm(username, password);
        const result = await LegacyNetworkMemberWithFactories_1._confirmSignUp(dependencies, options, cognitoTokens, password, undefined);
        result.afterConfirmSignUp();
        return result;
    }
    static async _signUpByEmailOrPhone(login, inputPassword, inputOptions, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: login },
            { isArray: false, type: 'password', isRequired: false, value: inputPassword },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const password = (0, normalizeShortPassword_1.normalizeShortPassword)(inputPassword || (await (0, generatePassword_1.generatePassword)()), login);
        return userManagementService.initiateSignUpWithEmailOrPhone(login, password, messageParameters);
    }
    /**
     * @description Initiates sign up flow
     * @param username - arbitrary username, email or phoneNumber
     * @param password - is required if arbitrary username was provided.
     * It is optional and random one will be generated, if not provided when
     * email or phoneNumber was given as a username.
     * @param options - optional parameters with specified environment
     * @param messageParameters - optional parameters with specified welcome message
     * @returns token or, in case when arbitrary username was used, it returns
     * initialized instance of SDK
     */
    static async signUp(dependencies, login, password, inputOptions, messageParameters) {
        const { isUsername } = (0, user_management_1.validateUsername)(login);
        if (!isUsername) {
            return LegacyNetworkMemberWithFactories_1._signUpByEmailOrPhone(login, password, inputOptions, messageParameters);
        }
        return LegacyNetworkMemberWithFactories_1._signUpByUsernameAutoConfirm(dependencies, login, password, inputOptions, messageParameters);
    }
    /**
     * @description Completes sign up flow with already created did
     *       (as result created keys will be stored at the Affinity Wallet)
     * NOTE: no need calling this method in case of arbitrary username was given,
     *       as registration is already completed
     * NOTE: This method will throw an error if called for arbitrary username
     * @param keyParams - { ecnryptedSeed, password } - previously created keys to be storead at wallet.
     * @param token - Token returned by signUpWithExistsEntity method.
     * @param confirmationCode - OTP sent by AWS Cognito/SES.
     * @param options - optional parameters for BaseNetworkMember initialization
     * @returns initialized instance of SDK
     */
    static async confirmSignUpWithExistsEntity(dependencies, keyParams, signUpToken, confirmationCode, inputOptions) {
        ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.KeyParams, isRequired: true, value: keyParams },
            { isArray: false, type: 'string', isRequired: true, value: signUpToken },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const { cognitoTokens, shortPassword } = await userManagementService.completeSignUpForEmailOrPhone(signUpToken, confirmationCode);
        return LegacyNetworkMemberWithFactories_1._confirmSignUp(dependencies, options, cognitoTokens, shortPassword, keyParams);
    }
    static async _confirmSignUp(dependencies, options, cognitoUserTokens, shortPassword, keyParams) {
        if (!(keyParams === null || keyParams === void 0 ? void 0 : keyParams.encryptedSeed)) {
            const passwordHash = WalletStorageService_1.default.hashFromString(shortPassword);
            const registerResult = await LegacyNetworkMemberWithFactories_1._register(dependencies, options, passwordHash);
            const encryptedSeed = registerResult.encryptedSeed;
            keyParams = { password: passwordHash, encryptedSeed };
        }
        const keyManagementService = (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const { encryptionKey, updatedEncryptedSeed } = await keyManagementService.reencryptSeed(cognitoUserTokens.accessToken, keyParams, !options.otherOptions.skipBackupEncryptedSeed);
        await userManagementService.markRegistrationComplete(cognitoUserTokens);
        const userData = (0, getDidData_1.withDidData)({
            encryptedSeed: updatedEncryptedSeed,
            password: encryptionKey,
        });
        return new LegacyNetworkMemberWithFactories_1({ ...userData, cognitoUserTokens }, dependencies, options);
    }
    /**
     * @description Completes sign up flow
     * NOTE: no need calling this method in case of arbitrary username was given,
     *       as registration is already completed.
     * NOTE: this method will throw an error if called for arbitrary username
     * @param confirmationCode - OTP sent by AWS Cognito/SES.
     * @param options - optional parameters for BaseNetworkMember initialization
     * @returns initialized instance of SDK
     */
    static async confirmSignUp(dependencies, signUpToken, confirmationCode, inputOptions, key, passwordless) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: signUpToken },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const { cognitoTokens, shortPassword } = await userManagementService.completeSignUpForEmailOrPhone(signUpToken, confirmationCode, key, passwordless);
        const result = await LegacyNetworkMemberWithFactories_1._confirmSignUp(dependencies, options, cognitoTokens, shortPassword, undefined);
        await result.afterConfirmSignUp();
        return result;
    }
    async afterConfirmSignUp() {
        const { idToken } = this.cognitoUserTokens;
        if (this._options.otherOptions.issueSignupCredential) {
            const signedCredentials = await this._getSignupCredentials(idToken, this._options);
            await this.saveCredentials(signedCredentials);
        }
    }
    /**
     * @description Resends OTP for sign up flow
     * @param username - email/phoneNumber, registered and unconfirmed in Cognito
     * @param options - optional parameters with specified environment
     * @param messageParameters - optional parameters with specified welcome message
     */
    static async resendSignUpConfirmationCode(login, inputOptions, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: login },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        await userManagementService.resendSignUpByLogin(login, messageParameters);
    }
    /**
     * @description Initiates passwordless sign in of an existing user,
     * or signs up a new one, if user was not registered
     * @param username - email/phoneNumber, registered in Cognito
     * @param options - optional parameters with specified environment
     * @param messageParameters - optional parameters with specified welcome message
     * @returns token
     */
    static async signIn(login, inputOptions, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: login },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        // NOTE: This is a passwordless login/sign up flow,
        //       case when user signs up more often
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const doesConfirmedUserExist = await userManagementService.doesConfirmedUserExist(login);
        if (doesConfirmedUserExist) {
            return userManagementService.initiateLogInPasswordless(login, messageParameters);
        }
        else {
            const password = (0, normalizeShortPassword_1.normalizeShortPassword)(await (0, generatePassword_1.generatePassword)(), login);
            return userManagementService.initiateSignUpWithEmailOrPhone(login, password, messageParameters, options.accessApiKey, true);
        }
    }
    /**
     * @description Completes sign in
     * @param token - received from #signIn method
     * @param confirmationCode - OTP sent by AWS Cognito/SES
     * @param options - optional parameters for BaseNetworkMember initialization
     * @returns an object with a flag, identifying whether new account was created, and initialized instance of SDK
     */
    static async confirmSignIn(dependencies, token, confirmationCode, options) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: token },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: options },
        ]);
        // NOTE: loginToken = '{"ChallengeName":"CUSTOM_CHALLENGE","Session":"...","ChallengeParameters":{"USERNAME":"...","email":"..."}}'
        //       signUpToken = 'username::password'
        const isSignUpToken = token.split('::')[1] !== undefined;
        if (isSignUpToken) {
            const commonNetworkMember = await LegacyNetworkMemberWithFactories_1.confirmSignUp(dependencies, token, confirmationCode, options, options.accessApiKey, true);
            return { isNew: true, commonNetworkMember };
        }
        const commonNetworkMember = await LegacyNetworkMemberWithFactories_1.completeLoginChallenge(dependencies, token, confirmationCode, options);
        return { isNew: false, commonNetworkMember };
    }
    /**
     * @description Initiates change user password
     * @param oldPassword
     * @param newPassword
     * @param options - optional parameters with specified environment
     */
    async changePassword(oldPassword, newPassword, inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: oldPassword },
            { isArray: false, type: 'string', isRequired: true, value: newPassword },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        this.cognitoUserTokens = await userManagementService.changePassword(this.cognitoUserTokens, oldPassword, newPassword);
    }
    /**
     * @description Initiates change user attribute (email/phoneNumber) flow
     * @param newLogin - new email/phoneNumber
     * @param options - optional parameters with specified environment
     * @param messageParameters - optional parameters with specified welcome message
     */
    // NOTE: operation is used for change the attribute, not username. Consider renaming
    //       New email/phoneNumber can be useded as a username to login.
    async changeUsername(newLogin, inputOptions, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: newLogin },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        this.cognitoUserTokens = await userManagementService.initiateChangeLogin(this.cognitoUserTokens, newLogin, messageParameters);
    }
    /**
     * @description Completes change user attribute (email/phoneNumber) flow
     * @param newUsername - new email/phoneNumber
     * @param confirmationCode - OTP sent by AWS Cognito/SES
     * @param options - optional parameters with specified environment
     */
    async confirmChangeUsername(newLogin, confirmationCode, inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: newLogin },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        this.cognitoUserTokens = await userManagementService.completeChangeLogin(this.cognitoUserTokens, newLogin, confirmationCode);
    }
    /**
     * @description Creates a new instance of SDK by access token
     * @param accessToken
     * @param options - optional parameters for AffinityWallet initialization
     * @returns initialized instance of SDK
     */
    static async fromAccessToken(dependencies, accessToken, inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: false, value: accessToken },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const keyManagementService = await (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const userData = await keyManagementService.pullUserData(accessToken);
        const cognitoUserTokens = { accessToken };
        return new LegacyNetworkMemberWithFactories_1({ ...userData, cognitoUserTokens }, dependencies, options);
    }
    /**
     * @description Logins with access token of Cognito user registered in Affinity
     * @param options - optional parameters for AffinityWallet initialization
     * @returns initialized instance of SDK or throws `COR-9` UnprocessableEntityError,
     * if user is not logged in.
     */
    static async init(dependencies, inputOptions) {
        await ParametersValidator_1.ParametersValidator.validate([{ isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions }]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const cognitoUserTokens = userManagementService.readUserTokensFromSessionStorage();
        const keyManagementService = (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const userData = await keyManagementService.pullUserData(cognitoUserTokens.accessToken);
        return new LegacyNetworkMemberWithFactories_1({ ...userData, cognitoUserTokens }, dependencies, options);
    }
    static legacyConstructor(dependencies, password, encryptedSeed, inputOptions) {
        if (!encryptedSeed || !password) {
            // TODO: implement appropriate error wrapper
            throw new Error('`password` and `encryptedSeed` must be provided!');
        }
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userData = (0, getDidData_1.withDidData)({ encryptedSeed, password });
        return new LegacyNetworkMemberWithFactories_1({ ...userData, cognitoUserTokens: undefined }, dependencies, options);
    }
};
LegacyNetworkMemberWithFactories = LegacyNetworkMemberWithFactories_1 = __decorate([
    (0, tools_common_1.profile)()
], LegacyNetworkMemberWithFactories);
exports.LegacyNetworkMemberWithFactories = LegacyNetworkMemberWithFactories;
//# sourceMappingURL=LegacyNetworkMemberWithFactories.js.map