"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var NetworkMemberWithCognito_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkMemberWithCognito = void 0;
const tools_common_1 = require("@affinidi/tools-common");
const uuid_1 = require("uuid");
const WalletStorageService_1 = __importDefault(require("../services/WalletStorageService"));
const getDidData_1 = require("../shared/getDidData");
const shared_dto_1 = require("../dto/shared.dto");
const ParametersValidator_1 = require("../shared/ParametersValidator");
const getOptionsFromEnvironment_1 = require("../shared/getOptionsFromEnvironment");
const BaseNetworkMember_1 = require("./BaseNetworkMember");
const createUserManagementService_1 = require("../shared/createUserManagementService");
const generatePassword_1 = require("../shared/generatePassword");
const normalizeShortPassword_1 = require("../shared/normalizeShortPassword");
const SdkErrorFromCode_1 = __importDefault(require("../shared/SdkErrorFromCode"));
let NetworkMemberWithCognito = NetworkMemberWithCognito_1 = class NetworkMemberWithCognito extends BaseNetworkMember_1.BaseNetworkMember {
    constructor(userData, dependencies, options) {
        super(userData, dependencies, options);
        this._userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        this.cognitoUserTokens = userData.cognitoUserTokens;
    }
    static _isKeyParams(keyParamsOrOptions) {
        return !!keyParamsOrOptions && 'encryptedSeed' in keyParamsOrOptions;
    }
    static _shouldCallAfterConfirmSignUp(keyParamsOrOptions) {
        return !NetworkMemberWithCognito_1._isKeyParams(keyParamsOrOptions);
    }
    static async _createKeyParams(dependencies, options, shortPassword, keyParamsOrOptions) {
        if (NetworkMemberWithCognito_1._isKeyParams(keyParamsOrOptions)) {
            return keyParamsOrOptions;
        }
        return await NetworkMemberWithCognito_1._createSignUpKeys(dependencies, options, shortPassword, keyParamsOrOptions);
    }
    static _createKeyParamsOrOptionsValidator(keyParamsOrOptions) {
        if (!keyParamsOrOptions || NetworkMemberWithCognito_1._isKeyParams(keyParamsOrOptions)) {
            return { isArray: false, type: shared_dto_1.KeyParams, isRequired: false, value: keyParamsOrOptions };
        }
        return { isArray: false, type: shared_dto_1.KeyOptions, isRequired: true, value: keyParamsOrOptions };
    }
    static _validateKeyParamsOrOptions(keyParamsOrOptions) {
        if (NetworkMemberWithCognito_1._isKeyParams(keyParamsOrOptions)) {
            NetworkMemberWithCognito_1._validateKeys(keyParamsOrOptions);
        }
    }
    /**
     * @description Initiates passwordless login to Affinity
     * @param options - parameters with specified environment
     * @param login - email/phoneNumber, registered in Cognito
     * @param messageParameters - optional parameters with specified welcome message
     * @returns token
     */
    static async initiateLogInPasswordless(inputOptions, login, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: login },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        return await userManagementService.initiateLogInPasswordless(login, messageParameters);
    }
    /**
     * @description Completes login
     * @param options - parameters for BaseNetworkMember initialization
     * @param token - returned by initiateLogInPasswordless
     * @param confirmationCode - OTP sent by AWS Cognito/SES
     * @returns initialized instance of SDK
     */
    static async completeLogInPasswordless(dependencies, inputOptions, token, confirmationCode) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: token },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const keyManagementService = (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const cognitoUserTokens = await userManagementService.completeLogInPasswordless(token, confirmationCode);
        const userData = await keyManagementService.pullUserData(cognitoUserTokens.accessToken);
        return new NetworkMemberWithCognito_1({ ...userData, cognitoUserTokens }, dependencies, options);
    }
    /**
     * @description Signs out current user
     */
    async logOut() {
        const newTokens = await this._userManagementService.logOut(this.cognitoUserTokens);
        this.cognitoUserTokens = newTokens;
    }
    /**
     * @description Get user metadata info
     * @returns userCreateDate
     */
    async getInfo() {
        return this._userManagementService.getInfo(this.cognitoUserTokens);
    }
    /**
     * @description Initiates reset password flow
     * @param options - parameters with specified environment
     * @param login - email/phoneNumber, registered in Cognito
     * @param messageParameters - optional parameters with specified welcome message
     * @returns token to be used with completeForgotPassword
     */
    static async initiateForgotPassword(inputOptions, login, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: login },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        await userManagementService.initiateForgotPassword(login, messageParameters);
        return JSON.stringify({ login });
    }
    /**
     * @description Completes reset password flow
     * @param options - parameters with specified environment
     * @param forgotPasswordToken - token returned by initiateForgotPassword
     * @param confirmationCode - OTP sent by AWS Cognito/SES
     * @param newPassword - new password
     * @returns initialized instance of SDK
     */
    static async completeForgotPassword(dependencies, inputOptions, forgotPasswordToken, confirmationCode, newPassword) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: forgotPasswordToken },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
            { isArray: false, type: 'string', isRequired: true, value: newPassword },
        ]);
        const { login } = JSON.parse(forgotPasswordToken);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        await userManagementService.completeForgotPassword(login, confirmationCode, newPassword);
        return await NetworkMemberWithCognito_1._logInWithPassword(dependencies, options, login, newPassword);
    }
    /**
     * @description Logins to Affinity with login and password
     * @param options - optional parameters for BaseNetworkMember initialization
     * @param login - arbitrary username or email or phone number, registered in Cognito
     * @param password - password for Cognito user
     * @returns initialized instance of SDK
     */
    static async logInWithPassword(dependencies, inputOptions, username, password) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: username },
            { isArray: false, type: 'password', isRequired: true, value: password },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        return await NetworkMemberWithCognito_1._logInWithPassword(dependencies, options, username, password);
    }
    static async _logInWithPassword(dependencies, options, username, inputPassword) {
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const keyManagementService = (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const password = (0, normalizeShortPassword_1.normalizeShortPassword)(inputPassword, username);
        const cognitoUserTokens = await userManagementService.logInWithPassword(username, password);
        const userData = await keyManagementService.pullUserData(cognitoUserTokens.accessToken);
        return new NetworkMemberWithCognito_1({ ...userData, cognitoUserTokens }, dependencies, options);
    }
    /**
     * @description Logins to Affinity with refresh token
     * @param dependencies - static dependencies including platform crypto tools
     * @param inputOptions - optional parameters for BaseNetworkMember initialization
     * @param refreshToken - refresh token to login and get new refreshed tokens
     * @returns initialized instance of SDK
     */
    static async logInWithRefreshToken(dependencies, inputOptions, refreshToken) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: refreshToken },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        return await NetworkMemberWithCognito_1._logInWithRefreshToken(dependencies, options, refreshToken);
    }
    static async _logInWithRefreshToken(dependencies, options, refreshToken) {
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const keyManagementService = (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const cognitoUserTokens = await userManagementService.logInWithRefreshToken(refreshToken);
        const userData = await keyManagementService.pullUserData(cognitoUserTokens.accessToken);
        return new NetworkMemberWithCognito_1({ ...userData, cognitoUserTokens }, dependencies, options);
    }
    /**
     * @description Initiates sign up flow to Affinity wallet, optionally with already created did
     * @param inputOptiosn - parameters with specified environment
     * @param username - arbitrary username
     * @param inputPassword - password
     * @param keyParamsOrOptions (optional) - { encryptedSeed, password } - previously created keys to be stored at wallet
     * @returns initialized instance of SDK
     */
    static async signUpWithUsername(dependencies, inputOptions, username, inputPassword, keyParamsOrOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: username },
            { isArray: false, type: 'password', isRequired: true, value: inputPassword },
            NetworkMemberWithCognito_1._createKeyParamsOrOptionsValidator(keyParamsOrOptions),
        ]);
        NetworkMemberWithCognito_1._validateKeyParamsOrOptions(keyParamsOrOptions);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const password = (0, normalizeShortPassword_1.normalizeShortPassword)(inputPassword, username);
        const cognitoTokens = await userManagementService.signUpWithUsernameAndConfirm(username, password);
        return NetworkMemberWithCognito_1._confirmSignUp(dependencies, options, cognitoTokens, password, keyParamsOrOptions);
    }
    static async _initiateSignUpByEmailOrPhone(inputOptions, login, inputPassword, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: login },
            { isArray: false, type: 'password', isRequired: false, value: inputPassword },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const password = (0, normalizeShortPassword_1.normalizeShortPassword)(inputPassword || (await (0, generatePassword_1.generatePassword)()), login);
        return userManagementService.initiateSignUpWithEmailOrPhone(login, password, messageParameters);
    }
    /**
     * @description Initiates sign up flow
     * @param options - parameters with specified environment
     * @param email - email address
     * @param password (optional) - if not provided, a random password will be generated
     * @param messageParameters (optional) - parameters with specified welcome message
     * @returns token
     */
    static async initiateSignUpByEmail(inputOptions, email, password, messageParameters) {
        return NetworkMemberWithCognito_1._initiateSignUpByEmailOrPhone(inputOptions, email, password, messageParameters);
    }
    /**
     * @description Initiates sign up flow
     * @param options - parameters with specified environment
     * @param phone - phone number
     * @param password (optional) - if not provided, a random password will be generated
     * @param messageParameters (optional) - parameters with specified welcome message
     * @returns token
     */
    static async initiateSignUpByPhone(inputOptions, phone, password, messageParameters) {
        return NetworkMemberWithCognito_1._initiateSignUpByEmailOrPhone(inputOptions, phone, password, messageParameters);
    }
    /**
     * @description Completes sign up flow, optionally with already created did
     *       (as result created keys will be stored at the Affinity Wallet)
     * @param options - optional parameters for BaseNetworkMember initialization
     * @param token - Token returned by initiateSignUp method.
     * @param confirmationCode - OTP sent by AWS Cognito/SES.
     * @param keyParamsOrOptions (optional) - { encryptedSeed, password } - previously created keys to be stored at wallet.
     * @returns initialized instance of SDK
     */
    static async completeSignUp(dependencies, inputOptions, signUpToken, confirmationCode, keyParamsOrOptions, passwordLess) {
        ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: signUpToken },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
            NetworkMemberWithCognito_1._createKeyParamsOrOptionsValidator(keyParamsOrOptions),
        ]);
        NetworkMemberWithCognito_1._validateKeyParamsOrOptions(keyParamsOrOptions);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const { cognitoTokens, shortPassword } = await userManagementService.completeSignUpForEmailOrPhone(signUpToken, confirmationCode, options.accessApiKey, passwordLess);
        return NetworkMemberWithCognito_1._confirmSignUp(dependencies, options, cognitoTokens, shortPassword, keyParamsOrOptions);
    }
    static async _createSignUpKeys(dependencies, options, shortPassword, keyOptions) {
        const passwordHash = WalletStorageService_1.default.hashFromString(shortPassword);
        const registerResult = await NetworkMemberWithCognito_1._register(dependencies, options, passwordHash, keyOptions);
        const { encryptedSeed, didDocument } = registerResult;
        return { password: passwordHash, encryptedSeed, didDocument };
    }
    static async _confirmSignUp(dependencies, options, cognitoUserTokens, shortPassword, inputKeyParamsOrOptions) {
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const keyManagementService = (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const { password, encryptedSeed, didDocument } = await NetworkMemberWithCognito_1._createKeyParams(dependencies, options, shortPassword, inputKeyParamsOrOptions);
        const { encryptionKey, updatedEncryptedSeed } = await keyManagementService.reencryptSeed(cognitoUserTokens.accessToken, { password, encryptedSeed }, !options.otherOptions.skipBackupEncryptedSeed);
        const userData = (0, getDidData_1.withDidData)({
            encryptedSeed: updatedEncryptedSeed,
            password: encryptionKey,
            didDocument,
        });
        const result = new NetworkMemberWithCognito_1({ ...userData, cognitoUserTokens }, dependencies, options);
        await userManagementService.markRegistrationComplete(cognitoUserTokens);
        if (NetworkMemberWithCognito_1._shouldCallAfterConfirmSignUp(inputKeyParamsOrOptions)) {
            result.afterConfirmSignUp();
        }
        return result;
    }
    async afterConfirmSignUp() {
        const { idToken } = this.cognitoUserTokens;
        if (this._options.otherOptions.issueSignupCredential) {
            const { basicOptions: { env, keyStorageUrl, issuerUrl }, accessApiKey, } = this._options;
            const credentialOfferToken = await WalletStorageService_1.default.getCredentialOffer(idToken, keyStorageUrl, {
                env,
                accessApiKey,
            });
            const credentialOfferResponseToken = await this.createCredentialOfferResponseToken(credentialOfferToken);
            const signedCredentials = await WalletStorageService_1.default.getSignedCredentials(idToken, credentialOfferResponseToken, {
                accessApiKey,
                issuerUrl,
                keyStorageUrl,
            });
            await this.saveCredentials(signedCredentials);
        }
    }
    /**
     * @description Resends OTP for sign up flow
     * @param inputOptions - parameters with specified environment
     * @param signUpToken - token returned by `initiateSignUp...`
     * @param messageParameters (optional) - parameters with specified welcome message
     */
    static async resendSignUp(inputOptions, signUpToken, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: signUpToken },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        await userManagementService.resendSignUpByToken(signUpToken, messageParameters);
    }
    /**
     * @description Initiates passwordless sign in of an existing user,
     * or signs up a new one, if user was not registered
     * @param options - optional parameters with specified environment
     * @param username - email or phone number
     * @param messageParameters - optional parameters with specified welcome message
     * @returns token
     */
    static async initiateSignInPasswordless(inputOptions, login, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: login },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        // NOTE: This is a passwordless login/sign up flow,
        //       case when user signs up more often
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        try {
            return JSON.stringify({
                signInType: 'logIn',
                logInToken: await userManagementService.initiateLogInPasswordless(login, messageParameters),
            });
        }
        catch (err) {
            // COR-4 user not found
            if (err.code !== 'COR-4') {
                throw err;
            }
            const password = (0, normalizeShortPassword_1.normalizeShortPassword)(await (0, generatePassword_1.generatePassword)(), login);
            return JSON.stringify({
                signInType: 'signUp',
                signUpToken: await userManagementService.initiateSignUpWithEmailOrPhone(login, password, messageParameters, options.accessApiKey, true),
            });
        }
    }
    /**
     * @description Completes sign in
     * @param options - optional parameters for BaseNetworkMember initialization
     * @param token - received from #signIn method
     * @param confirmationCode - OTP sent by AWS Cognito/SES
     * @returns an object with a flag, identifying whether new account was created, and initialized instance of SDK
     */
    static async completeSignInPasswordless(dependencies, options, signInToken, confirmationCode) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: options },
            { isArray: false, type: 'string', isRequired: true, value: signInToken },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
        ]);
        let token;
        try {
            token = JSON.parse(signInToken);
        }
        catch (error) {
            throw new SdkErrorFromCode_1.default('COR-35', {}, error);
        }
        switch (token.signInType) {
            case 'logIn':
                return {
                    isNew: false,
                    wallet: await this.completeLogInPasswordless(dependencies, options, token.logInToken, confirmationCode),
                };
            case 'signUp':
                return {
                    isNew: true,
                    wallet: await this.completeSignUp(dependencies, options, token.signUpToken, confirmationCode, null, true),
                };
            default:
                throw new SdkErrorFromCode_1.default('COR-35', { message: `Incorrect token type '${token.signInType}'` });
        }
    }
    /**
     * @description Initiates login of an existing user with `Truecaller` token,
     * or signs up a new one, if user was not registered
     * @param inputOptions - optional parameters with specified environment
     * @param profileTrueCaller - phone number
     * @param dependencies - static dependencies(cryptographic tools and data to format events logging)
     * @returns wallet
     */
    static async signInWithProfile(inputOptions, profileTrueCaller, dependencies) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.TokenTrueCaller, isRequired: true, value: profileTrueCaller },
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userManagementService = (0, createUserManagementService_1.createUserManagementService)(options);
        const keyManagementService = (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const isValid = await userManagementService.validateProfile(profileTrueCaller);
        if (!isValid) {
            throw new Error(`Token is not valid!`);
        }
        const { phoneNumber } = await userManagementService.parseAndValidatePayload(profileTrueCaller);
        const userExists = await userManagementService.doesConfirmedUserExist(phoneNumber);
        if (userExists) {
            // logIn
            const cognitoUserTokens = await userManagementService.logInWithProfile(phoneNumber, profileTrueCaller, inputOptions.env, false);
            const userData = await keyManagementService.pullUserData(cognitoUserTokens.accessToken);
            return {
                isNew: false,
                wallet: new NetworkMemberWithCognito_1({ ...userData, cognitoUserTokens }, dependencies, options),
            };
        }
        else {
            // signup
            const newUsername = (0, uuid_1.v4)();
            const password = (0, normalizeShortPassword_1.normalizeShortPassword)(await (0, generatePassword_1.generatePassword)(), newUsername);
            const cognitoTokens = await userManagementService.signUpWithProfile(newUsername, password, profileTrueCaller, inputOptions.env);
            return {
                isNew: true,
                wallet: await NetworkMemberWithCognito_1._confirmSignUp(dependencies, options, cognitoTokens, password),
            };
        }
    }
    /**
     * @description Initiates change user password   * @param options - parameters with specified environment
     * @param oldPassword
     * @param newPassword
     */
    async changePassword(oldPassword, newPassword) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: oldPassword },
            { isArray: false, type: 'string', isRequired: true, value: newPassword },
        ]);
        this.cognitoUserTokens = await this._userManagementService.changePassword(this.cognitoUserTokens, oldPassword, newPassword);
    }
    async _initiateChangeEmailOrPhone(newLogin, messageParameters) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: newLogin },
            { isArray: false, type: shared_dto_1.MessageParameters, isRequired: false, value: messageParameters },
        ]);
        this.cognitoUserTokens = await this._userManagementService.initiateChangeLogin(this.cognitoUserTokens, newLogin, messageParameters);
        return JSON.stringify({ newLogin });
    }
    /**
     * @description Initiates change user attribute (email) flow
     * @param email - new email
     * @param messageParameters - optional parameters with specified welcome message
     * @returns token to be used with completeChangeEmailOrPhone
     */
    async initiateChangeEmail(email, messageParameters) {
        return this._initiateChangeEmailOrPhone(email, messageParameters);
    }
    /**
     * @description Initiates change user attribute (email) flow
     * @param phone - new phone
     * @param messageParameters - optional parameters with specified welcome message
     * @returns token to be used with completeChangeEmailOrPhone
     */
    async initiateChangePhone(phone, messageParameters) {
        return this._initiateChangeEmailOrPhone(phone, messageParameters);
    }
    /**
     * @description Completes change user attribute (email/phoneNumber) flow
     * @param changeEmailOrPhoneToken - token returned by initiateChangePhone or initiateChangeEmail method
     * @param confirmationCode - OTP sent by AWS Cognito/SES
     */
    async completeChangeEmailOrPhone(changeEmailOrPhoneToken, confirmationCode) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: 'string', isRequired: true, value: changeEmailOrPhoneToken },
            { isArray: false, type: 'confirmationCode', isRequired: true, value: confirmationCode },
        ]);
        const { newLogin } = JSON.parse(changeEmailOrPhoneToken);
        this.cognitoUserTokens = await this._userManagementService.completeChangeLogin(this.cognitoUserTokens, newLogin, confirmationCode);
    }
    /**
     * @description Serialized the essential data to string that could be later used by `deserialize`
     * @returns string
     */
    serializeSession() {
        return JSON.stringify(this.cognitoUserTokens);
    }
    /**
     * @description Creates a new instance of SDK from string returned by `serialize`
     * @param options - parameters for AffinityWallet initialization
     * @param serializedNetworkMember
     * @returns initialized instance of SDK
     */
    static async deserializeSession(dependencies, inputOptions, serializedSession) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: serializedSession },
        ]);
        const cognitoUserTokens = JSON.parse(serializedSession);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const keyManagementService = await (0, BaseNetworkMember_1.createKeyManagementService)(options);
        const userData = await keyManagementService.pullUserData(cognitoUserTokens.accessToken);
        return new NetworkMemberWithCognito_1({ ...userData, cognitoUserTokens }, dependencies, options);
    }
};
NetworkMemberWithCognito = NetworkMemberWithCognito_1 = __decorate([
    (0, tools_common_1.profile)()
], NetworkMemberWithCognito);
exports.NetworkMemberWithCognito = NetworkMemberWithCognito;
//# sourceMappingURL=NetworkMemberWithCognito.js.map