import { KeyOptions, SdkOptions } from '../dto/shared.dto';
import { ParsedOptions } from '../shared/getOptionsFromEnvironment';
import { BaseNetworkMember, StaticDependencies, ConstructorUserData } from './BaseNetworkMember';
export declare class NetworkMemberWithoutCognito extends BaseNetworkMember {
    constructor(userData: ConstructorUserData, dependencies: StaticDependencies, options: ParsedOptions);
    /**
     * @description Creates DID and anchors it
     * 1. generate seed/keys (available key types rsa, bbs and ecdsa as a default)
     * 2. build DID document
     * 3. sign DID document
     * 4. store DID document in IPFS
     * 5. anchor DID with DID document ID from IPFS
     * @param dependencies - static dependencies
     * @param password - encryption key which will be used to encrypt randomly created seed/keys pair
     * @param inputOptions - optional parameter { registryUrl: 'https://affinity-registry.apse1.dev.affinidi.io' }
     * @param keyOptions - list of key types (rsa and bbs allowed, ecdsa - default)
     * @returns
     *
     * did - hash from public key (your decentralized ID)
     *
     * encryptedSeed - seed is encrypted by provided password. Seed - it's a source to derive your keys
     */
    static createWallet(dependencies: StaticDependencies, inputOptions: SdkOptions, password: string, keyOptions?: KeyOptions): Promise<NetworkMemberWithoutCognito>;
    /**
     * @description Initilizes instance of SDK from seed
     * @param dependencies - static dependencies
     * @param inputOptions - parameter { registryUrl: 'https://affinity-registry.apse1.dev.affinidi.io' }
     * @param encryptedSeed - seed for derive keys in string hex format
     * @param password - optional password, will be generated, if not provided
     * @param accountNumber - optional parameter
     * @returns initialized instance of SDK
     */
    static openWalletByEncryptedSeed(dependencies: StaticDependencies, inputOptions: SdkOptions, encryptedSeed: string, password: string, accountNumber?: number): Promise<NetworkMemberWithoutCognito>;
}
