"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NetworkMemberWithoutCognito_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkMemberWithoutCognito = void 0;
const tools_common_1 = require("@affinidi/tools-common");
const shared_dto_1 = require("../dto/shared.dto");
const getDidData_1 = require("../shared/getDidData");
const getOptionsFromEnvironment_1 = require("../shared/getOptionsFromEnvironment");
const ParametersValidator_1 = require("../shared/ParametersValidator");
const BaseNetworkMember_1 = require("./BaseNetworkMember");
let NetworkMemberWithoutCognito = NetworkMemberWithoutCognito_1 = class NetworkMemberWithoutCognito extends BaseNetworkMember_1.BaseNetworkMember {
    constructor(userData, dependencies, options) {
        super(userData, dependencies, options);
    }
    /**
     * @description Creates DID and anchors it
     * 1. generate seed/keys (available key types rsa, bbs and ecdsa as a default)
     * 2. build DID document
     * 3. sign DID document
     * 4. store DID document in IPFS
     * 5. anchor DID with DID document ID from IPFS
     * @param dependencies - static dependencies
     * @param password - encryption key which will be used to encrypt randomly created seed/keys pair
     * @param inputOptions - optional parameter { registryUrl: 'https://affinity-registry.apse1.dev.affinidi.io' }
     * @param keyOptions - list of key types (rsa and bbs allowed, ecdsa - default)
     * @returns
     *
     * did - hash from public key (your decentralized ID)
     *
     * encryptedSeed - seed is encrypted by provided password. Seed - it's a source to derive your keys
     */
    static async createWallet(dependencies, inputOptions, password, keyOptions) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: password },
            { isArray: false, type: shared_dto_1.KeyOptions, isRequired: false, value: keyOptions },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        const userData = await NetworkMemberWithoutCognito_1._register(dependencies, options, password, keyOptions);
        return new NetworkMemberWithoutCognito_1({ ...userData, password }, dependencies, options);
    }
    /**
     * @description Initilizes instance of SDK from seed
     * @param dependencies - static dependencies
     * @param inputOptions - parameter { registryUrl: 'https://affinity-registry.apse1.dev.affinidi.io' }
     * @param encryptedSeed - seed for derive keys in string hex format
     * @param password - optional password, will be generated, if not provided
     * @param accountNumber - optional parameter
     * @returns initialized instance of SDK
     */
    static async openWalletByEncryptedSeed(dependencies, inputOptions, encryptedSeed, password, accountNumber) {
        await ParametersValidator_1.ParametersValidator.validate([
            { isArray: false, type: shared_dto_1.SdkOptions, isRequired: true, value: inputOptions },
            { isArray: false, type: 'string', isRequired: true, value: encryptedSeed },
            { isArray: false, type: 'string', isRequired: true, value: password },
        ]);
        const options = (0, getOptionsFromEnvironment_1.getOptionsFromEnvironment)(inputOptions);
        return new NetworkMemberWithoutCognito_1((0, getDidData_1.withDidData)({ password, encryptedSeed, accountNumber }), dependencies, options);
    }
};
NetworkMemberWithoutCognito = NetworkMemberWithoutCognito_1 = __decorate([
    (0, tools_common_1.profile)()
], NetworkMemberWithoutCognito);
exports.NetworkMemberWithoutCognito = NetworkMemberWithoutCognito;
//# sourceMappingURL=NetworkMemberWithoutCognito.js.map