"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Util = void 0;
const common_1 = require("@affinidi/common");
const user_management_1 = require("@affinidi/user-management");
const ParametersValidator_1 = require("../shared/ParametersValidator");
const randomBytes_1 = require("../shared/randomBytes");
const _defaultConfig_1 = require("../_defaultConfig");
const common_2 = require("@affinidi/common");
exports.Util = {
    /**
     * @description Parses JWT and returns DID
     * @param jwt
     * @returns DID of entity who signed JWT
     */
    getDidFromToken: (jwt) => {
        // await ParametersValidator.validateSync(
        //   [
        //     { isArray: false, type: 'jwt', isRequired: true, value: jwt }
        //   ]
        // )
        return common_1.JwtService.getDidFromToken(jwt);
    },
    /**
     * @description Returns hex of public key from DID document
     * @param didDocument - user's DID document
     * @returns public key hex
     */
    getPublicKeyHexFromDidDocument: (didDocument) => {
        // await ParametersValidator.validateSync(
        //   [
        //     { isArray: false, type: 'object', isRequired: true, value: didDocument }
        //   ]
        // )
        // TODO: review: in general case - need to find section at didDocument.publicKey where id === keyId
        const { publicKeyHex } = didDocument.publicKey[0];
        return publicKeyHex;
    },
    /**
     * @description Returns public key from DID document
     * @param didDocument - user's DID document
     * @returns publicKey
     */
    getPublicKeyFromDidDocument: (didDocument) => {
        var _a, _b;
        const keySection = (_a = didDocument.publicKey) === null || _a === void 0 ? void 0 : _a[0]; // old form of did docs contains publicKey[0].publicKeyHex
        const methodSection = (_b = didDocument.verificationMethod) === null || _b === void 0 ? void 0 : _b[0]; // new form of did docs contains verificationMethod[0].publicKeyBase58
        if (keySection === null || keySection === void 0 ? void 0 : keySection.publicKeyPem)
            return Buffer.from(keySection.publicKeyPem);
        if (keySection === null || keySection === void 0 ? void 0 : keySection.publicKeyBase58)
            return Buffer.from(keySection.publicKeyBase58);
        if (keySection === null || keySection === void 0 ? void 0 : keySection.publicKeyHex)
            return Buffer.from(keySection.publicKeyHex, 'hex');
        if (methodSection === null || methodSection === void 0 ? void 0 : methodSection.publicKeyBase58)
            return (0, common_2.decodeBase58)(methodSection === null || methodSection === void 0 ? void 0 : methodSection.publicKeyBase58);
    },
    /**
     * @description Generates random seed from which keys could be derived
     */
    generateSeed: async (didMethod = _defaultConfig_1.DEFAULT_DID_METHOD) => {
        await ParametersValidator_1.ParametersValidator.validate([{ isArray: false, type: 'didMethod', isRequired: true, value: didMethod }]);
        let seed;
        switch (didMethod) {
            case 'jolo':
                seed = await (0, randomBytes_1.randomBytes)(32);
                break;
            default:
                seed = await (0, randomBytes_1.randomBytes)(32);
        }
        return seed;
    },
    /**
     * @description Parses JWT token (request and response tokens of share and offer flows)
     * @param token - JWT
     * @returns parsed object from JWT
     */
    fromJWT: (token) => {
        // await ParametersValidator.validateSync(
        //   [
        //     { isArray: false, type: 'jwt', isRequired: true, value: token }
        //   ]
        // )
        return common_1.JwtService.fromJWT(token);
    },
    /**
     * Simple check if token has JWT structure
     * @param token
     */
    isJWT: (token) => {
        return common_1.JwtService.isJWT(token);
    },
    getLoginType: (login) => {
        const { isEmailValid, isPhoneNumberValid } = (0, user_management_1.validateUsername)(login);
        if (isEmailValid) {
            return 'email';
        }
        if (isPhoneNumberValid) {
            return 'phone';
        }
        return 'username';
    },
};
//# sourceMappingURL=Util.js.map