import { SUPPORTED_DID_METHODS } from '../_defaultConfig';
import { DocumentLoader, KeyManager } from '@affinidi/common';
import { CognitoIdentityProviderClient } from '@affinidi/user-management';
export declare type Env = 'dev' | 'staging' | 'prod';
export declare type DidMethod = typeof SUPPORTED_DID_METHODS[number];
export declare const DID_METHOD: RegExp;
export declare const SIGNATURE_ALGO: RegExp;
export declare const DID: RegExp;
export declare const PASSWORD: RegExp;
export declare const JWT: RegExp;
export declare const COGNITO_CONFIRMATION_CODE: RegExp;
export declare class SdkOptions {
    registryUrl?: string;
    issuerUrl?: string;
    verifierUrl?: string;
    keyStorageUrl?: string;
    affinidiVaultUrl?: string;
    revocationUrl?: string;
    didMethod?: DidMethod;
    webDomain?: string;
    env: Env;
    phoneIssuerBasePath?: string;
    emailIssuerBasePath?: string;
    skipBackupEncryptedSeed?: boolean;
    skipBackupCredentials?: boolean;
    apiKey?: string;
    accessApiKey?: string;
    isProfilerActive?: boolean;
    metricsUrl?: string;
    storageRegion?: string;
    issueSignupCredential?: boolean;
    userPoolId?: string;
    clientId?: string;
    region?: string;
    origin?: string;
    skipAnchoringForElemMethod?: boolean;
    resolveLocallyElemMethod?: boolean;
    resolveKeyLocally?: boolean;
    /**
     * document loader invoked before the main document loader.
     * If it returns result - the result is considered as loaded document and used as a result.
     * If it returns an undefined - the main document Loader is invoked with the same params.
     */
    beforeDocumentLoader?: DocumentLoader;
    cognitoProviderClient?: CognitoIdentityProviderClient;
    keyManager?: KeyManager;
    tenantToken?: string;
}
export declare class MessageParameters {
    message: string;
    subject?: string;
    htmlMessage?: string;
}
export declare class KeyParams {
    encryptedSeed: string;
    password: string;
    didDocument?: any;
}
export declare type KeyAlgorithmType = 'rsa' | 'bbs' | 'ecdsa';
export declare class KeyOptions {
    keyTypes: KeyAlgorithmType[];
}
export declare type KeyParamsOrOptions = KeyParams | KeyOptions;
export declare class CognitoUserTokens {
    accessToken?: string;
    idToken?: string;
    refreshToken?: string;
    expiresIn?: number;
}
export declare class SdkOptionsWithCongitoSetup extends SdkOptions {
    userPoolId?: string;
    clientId?: string;
}
export declare class CredentialParams {
    type: string;
    context: unknown[];
    credentialSubject: unknown;
    expiresAt?: string;
}
export declare class SignedCredential {
    claim?: unknown;
    issued?: string;
    expires?: string;
    '@context': unknown[];
    id: string;
    name?: string;
    issuer: string | {
        id: string;
    };
    issuanceDate?: string;
    expirationDate?: string;
    type: string[];
    credentialSubject?: unknown;
    proof: Proof | Proof[];
    credentialStatus?: unknown;
    verifiableCredential?: unknown[];
    credentialSchema?: unknown;
    refreshService?: unknown;
    termsOfUse?: unknown[];
    evidence?: unknown[];
}
export declare class CredentialRequirement {
    type: string[];
    constraints?: string[];
}
export declare class JwtOptions {
    audienceDid?: string;
    expiresAt?: string;
    nonce?: string;
    callbackUrl?: string;
}
export declare class OfferedCredential {
    type: string;
    renderInfo?: unknown;
}
export declare class Proof {
    type: string;
}
export declare class ClaimMetadata {
    context?: unknown;
    name?: string;
    type?: string[];
    claimInterface?: unknown;
}
export declare class SignCredentialOptionalInput {
    requesterDid?: string;
    credentialOfferResponseToken?: string;
}
export declare class FetchCredentialsPaginationOptions {
    skip?: number;
    limit?: number;
}
export declare class TokenTrueCaller {
    payload: string;
    signature: string;
    signatureAlgorithm: string;
    avatarUrl?: string | null;
    city?: string | null;
    companyName?: string | null;
    countryCode?: string | null;
    email?: string | null;
    facebookId?: string | null;
    firstName?: string;
    gender?: string;
    isAmbassador?: boolean;
    isBusiness?: boolean;
    isVerified?: boolean;
    jobTitle?: string | null;
    lastName?: string;
    phoneNumber?: string;
    requestNonce?: string;
    street?: string | null;
    successful?: boolean;
    twitterId?: string | null;
    url?: string | null;
    zipcode?: string | null;
}
