"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCognitolessWalletFactories = void 0;
const NetworkMemberWithoutCognito_1 = require("../CommonNetworkMember/NetworkMemberWithoutCognito");
const createCognitolessWalletFactories = (platformCryptographyTools, eventComponent) => {
    const dependencies = { platformCryptographyTools, eventComponent };
    return {
        /**
         * @description Generates a new DID and creates a new instance of SDK using password
         * @param inputOptions - parameters with specified environment
         * @param inputPassword - password
         * @param keyOptions - specifies available wallet key types (allowed rsa and bbs; ecdsa - default, no need to specify)
         * @returns initialized instance of SDK
         */
        createWallet: (inputOptions, inputPassword, keyOptions) => {
            return NetworkMemberWithoutCognito_1.NetworkMemberWithoutCognito.createWallet(dependencies, inputOptions, inputPassword, keyOptions);
        },
        /**
         * @description Initilizes instance of SDK from seed
         * @param inputOptions - parameters with specified environment
         * @param encryptedSeed - encrypted seed
         * @param password - password
         * @param accountNumber - account number is an optional parameter to derive custom account `keys`/`did` from the root `seed`
         * @returns initialized instance of SDK
         */
        openWalletByEncryptedSeed: (inputOptions, encryptedSeed, password, accountNumber) => {
            return NetworkMemberWithoutCognito_1.NetworkMemberWithoutCognito.openWalletByEncryptedSeed(dependencies, inputOptions, encryptedSeed, password, accountNumber);
        },
    };
};
exports.createCognitolessWalletFactories = createCognitolessWalletFactories;
//# sourceMappingURL=cognitolessWalletFactories.js.map