"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLegacyWalletFactories = void 0;
const LegacyNetworkMemberWithFactories_1 = require("../CommonNetworkMember/LegacyNetworkMemberWithFactories");
const Util_1 = require("../CommonNetworkMember/Util");
const createLegacyWalletFactories = (platformCryptographyTools, eventComponent) => {
    const dependencies = { platformCryptographyTools, eventComponent };
    return {
        /**
         * @description Checks if registration for the user was completed
         * @param username - a valid email, phone number or arbitrary username
         * @param options - object with environment, dev is default { env: 'dev' }
         * @returns `true` if user is uncofirmed in Cognito, and `false` otherwise.
         * @deprecated
         */
        isUserUnconfirmed: (username, options) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.isUserUnconfirmed(username, options);
        },
        /**
         * @description Initilizes instance of SDK from seed
         * @param seedHexWithMethod - seed for derive keys in string hex format
         * @param options - optional parameter { registryUrl: 'https://affinity-registry.apse1.dev.affinidi.io' }
         * @param password - optional password, will be generated, if not provided
         * @returns initialized instance of SDK
         * @deprecated
         */
        fromSeed: (seedHexWithMethod, options, password = null) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.fromSeed(dependencies, seedHexWithMethod, options, password);
        },
        /**
         * @description Parses JWT and returns DID
         * @param jwt
         * @returns DID of entity who signed JWT
         * @deprecated use `Util` instead
         */
        getDidFromToken: (jwt) => {
            return Util_1.Util.getDidFromToken(jwt);
        },
        /**
         * @description Creates DID and anchors it
         * 1. generate seed/keys
         * 2. build DID document
         * 3. sign DID document
         * 4. store DID document in IPFS
         * 5. anchor DID with DID document ID from IPFS
         * @param password - encryption key which will be used to encrypt randomly created seed/keys pair
         * @param options - optional parameter { registryUrl: 'https://affinity-registry.apse1.dev.affinidi.io' }
         * @returns
         *
         * did - hash from public key (your decentralized ID)
         *
         * encryptedSeed - seed is encrypted by provided password. Seed - it's a source to derive your keys
         * @deprecated Use `createWallet` instead
         */
        register: (password, options) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.register(dependencies, options, password);
        },
        /**
         * @deprecated
         */
        anchorDid: (encryptedSeed, password, didDocument, nonce, options) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.anchorDid(encryptedSeed, password, didDocument, nonce, options);
        },
        /**
         * @description Initiates passwordless login to Affinity
         * @param username - email/phoneNumber, registered in Cognito
         * @param options - optional parameters with specified environment
         * @param messageParameters - optional parameters with specified welcome message
         * @returns token
         * @deprecated use `initiateLogInPasswordless` instead
         */
        passwordlessLogin: (login, options, messageParameters) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.passwordlessLogin(login, options, messageParameters);
        },
        /**
         * @description Completes login
         * @param token - received from #passwordlessLogin method
         * @param confirmationCode - OTP sent by AWS Cognito/SES
         * @param options - optional parameters for CommonNetworkMember initialization
         * @returns initialized instance of SDK
         * @deprecated use `completeLogInPasswordless` instead
         */
        completeLoginChallenge: (token, confirmationCode, inputOptions) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.completeLoginChallenge(dependencies, token, confirmationCode, inputOptions);
        },
        /**
         * @description Initiates reset password flow
         * @param username - email/phoneNumber, registered in Cognito
         * @param options - optional parameters with specified environment
         * @param messageParameters - optional parameters with specified welcome message
         * @deprecated use `initiateForgotPassword` instead
         */
        forgotPassword: (login, options, messageParameters) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.forgotPassword(login, options, messageParameters);
        },
        /**
         * @description Completes reset password flow
         * @param username - email/phoneNumber, registered in Cognito
         * @param confirmationCode - OTP sent by AWS Cognito/SES
         * @param newPassword - new password
         * @param options - optional parameters with specified environment
         * @deprecated use `completeForgotPassword` instead
         */
        forgotPasswordSubmit: (login, confirmationCode, newPassword, options) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.forgotPasswordSubmit(login, confirmationCode, newPassword, options);
        },
        /**
         * @description Logins to Affinity with login and password
         * @param username - email/phoneNumber, registered in Cognito
         * @param password - password for Cognito user
         * @param options - optional parameters for CommonNetworkMember initialization
         * @returns initialized instance of SDK
         * @deprecated use `logInWithPassword` instead
         */
        fromLoginAndPassword: (username, password, inputOptions) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.fromLoginAndPassword(dependencies, username, password, inputOptions);
        },
        /**
         * @description Initiates sign up flow to Affinity wallet with already created did
         * @param keyParams - { ecnryptedSeed, password } - previously created keys to be storead at wallet.
         * @param username - arbitrary username, email or phoneNumber
         * @param password - is required if arbitrary username was provided.
         * It is optional and random one will be generated, if not provided when
         * email or phoneNumber was given as a username.
         * @param options - optional parameters with specified environment
         * @param messageParameters - optional parameters with specified welcome message
         * @returns token or, in case when arbitrary username was used, it returns
         * initialized instance of SDK
         * @deprecated use `signUpWithUsername`, `initiateSignUpByEmail` or `initiateSignUpByPhone` instead
         */
        signUpWithExistsEntity: (keyParams, login, password, options, messageParameters) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.signUpWithExistsEntity(dependencies, keyParams, login, password, options, messageParameters);
        },
        /**
         * @description Initiates sign up flow
         * @param username - arbitrary username, email or phoneNumber
         * @param password - is required if arbitrary username was provided.
         * It is optional and random one will be generated, if not provided when
         * email or phoneNumber was given as a username.
         * @param options - optional parameters with specified environment
         * @param messageParameters - optional parameters with specified welcome message
         * @returns token or, in case when arbitrary username was used, it returns
         * initialized instance of SDK
         * @deprecated use `signUpWithUsername`, `initiateSignUpByEmail` or `initiateSignUpByPhone` instead
         */
        signUp: (login, password, options, messageParameters) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.signUp(dependencies, login, password, options, messageParameters);
        },
        /**
         * @description Completes sign up flow with already created did
         *       (as result created keys will be stored at the Affinity Wallet)
         * NOTE: no need calling this method in case of arbitrary username was given,
         *       as registration is already completed
         * NOTE: This method will throw an error if called for arbitrary username
         * @param keyParams - { ecnryptedSeed, password } - previously created keys to be storead at wallet.
         * @param token - Token returned by signUpWithExistsEntity method.
         * @param confirmationCode - OTP sent by AWS Cognito/SES.
         * @param options - optional parameters for CommonNetworkMember initialization
         * @returns initialized instance of SDK
         * @deprecated use `completeSignUp` instead
         */
        confirmSignUpWithExistsEntity: (keyParams, signUpToken, confirmationCode, options) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.confirmSignUpWithExistsEntity(dependencies, keyParams, signUpToken, confirmationCode, options);
        },
        /**
         * @description Completes sign up flow
         * NOTE: no need calling this method in case of arbitrary username was given,
         *       as registration is already completed.
         * NOTE: this method will throw an error if called for arbitrary username
         * @param confirmationCode - OTP sent by AWS Cognito/SES.
         * @param options - optional parameters for CommonNetworkMember initialization
         * @returns initialized instance of SDK
         * @deprecated use `completeSignUp` instead
         */
        confirmSignUp: (signUpToken, confirmationCode, options) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.confirmSignUp(dependencies, signUpToken, confirmationCode, options);
        },
        /**
         * @description Resends OTP for sign up flow
         * @param username - email/phoneNumber, registered and unconfirmed in Cognito
         * @param options - optional parameters with specified environment
         * @param messageParameters - optional parameters with specified welcome message
         * @deprecated use `resendSignUp` instead
         */
        resendSignUpConfirmationCode: (login, options, messageParameters) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.resendSignUpConfirmationCode(login, options, messageParameters);
        },
        /**
         * @description Initiates passwordless sign in of an existing user,
         * or signs up a new one, if user was not registered
         * @param username - email/phoneNumber, registered in Cognito
         * @param options - optional parameters with specified environment
         * @param messageParameters - optional parameters with specified welcome message
         * @returns token
         * @deprecated use `initiateSignInPasswordless` instead
         */
        signIn: (login, options, messageParameters) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.signIn(login, options, messageParameters);
        },
        /**
         * @description Completes sign in
         * @param token - received from #signIn method
         * @param confirmationCode - OTP sent by AWS Cognito/SES
         * @param options - optional parameters for CommonNetworkMember initialization
         * @returns an object with a flag, identifying whether new account was created, and initialized instance of SDK
         * @deprecated use `completeSignInPasswordless` instead
         */
        confirmSignIn: (token, confirmationCode, options) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.confirmSignIn(dependencies, token, confirmationCode, options);
        },
        /**
         * @description Generates random seed from which keys could be derived
         * @deprecated use `Util` instead
         */
        generateSeed: (didMethod) => {
            return Util_1.Util.generateSeed(didMethod);
        },
        /**
         * @description Parses JWT token (request and response tokens of share and offer flows)
         * @param token - JWT
         * @returns parsed object from JWT
         * @deprecated use `Util` instead
         */
        fromJWT: (token) => {
            return Util_1.Util.fromJWT(token);
        },
        /**
         * @description Creates a new instance of SDK by access token
         * @param accessToken
         * @param options - optional parameters for AffinityWallet initialization
         * @returns initialized instance of SDK
         * @deprecated use `deserialize` instead
         */
        fromAccessToken: (accessToken, options) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.fromAccessToken(dependencies, accessToken, options);
        },
        /**
         * @description Logins with access token of Cognito user registered in Affinity
         * @param options - optional parameters for AffinityWallet initialization
         * @returns initialized instance of SDK or throws `COR-9` UnprocessableEntityError,
         * if user is not logged in.
         * @deprecated manage state storage and use `serialize`/`deserialize` instead
         */
        init: (options) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.init(dependencies, options);
        },
        /**
         * @deprecated use `openWalletByEncryptedSeed` instead
         */
        legacyConstructor: (password, encryptedSeed, inputOptions) => {
            return LegacyNetworkMemberWithFactories_1.LegacyNetworkMemberWithFactories.legacyConstructor(dependencies, password, encryptedSeed, inputOptions);
        },
    };
};
exports.createLegacyWalletFactories = createLegacyWalletFactories;
//# sourceMappingURL=legacyWalletFactories.js.map