"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createV6WalletFactories = exports.createV5CompatibleWalletFactories = void 0;
const cognitoWalletFactories_1 = require("./cognitoWalletFactories");
const cognitolessWalletFactories_1 = require("./cognitolessWalletFactories");
const legacyWalletFactories_1 = require("./legacyWalletFactories");
const publicToolsFactories_1 = require("./publicToolsFactories");
/**
 * Turns any arrow function into a constructor of its return value accepting the same arguments
 * @param inputFunction Original function (e.g. `(arg: string) => ({ arg })`)
 * @returns Constructor (e.g. `C` such that `new C('abc') === { arg: 'abc' }`)
 */
const createConstructor = (inputFunction) => {
    const f = inputFunction;
    return function (...args) {
        return f(...args);
    };
};
const createV5CompatibleWalletFactories = (platformCryptographyTools, component) => {
    const { legacyConstructor, ...legacyFactories } = (0, legacyWalletFactories_1.createLegacyWalletFactories)(platformCryptographyTools, component);
    const cognitoFactories = (0, cognitoWalletFactories_1.createCognitoWalletFactories)(platformCryptographyTools, component);
    const cognitolessFactories = (0, cognitolessWalletFactories_1.createCognitolessWalletFactories)(platformCryptographyTools, component);
    const publicToolsFactories = (0, publicToolsFactories_1.createPublicToolsFactories)(platformCryptographyTools, component);
    return Object.assign(createConstructor(legacyConstructor), {
        ...cognitoFactories,
        ...cognitolessFactories,
        ...publicToolsFactories,
        ...legacyFactories,
    });
};
exports.createV5CompatibleWalletFactories = createV5CompatibleWalletFactories;
const createV6WalletFactories = (platformCryptographyTools, component) => {
    const cognitoFactories = (0, cognitoWalletFactories_1.createCognitoWalletFactories)(platformCryptographyTools, component);
    const cognitolessFactories = (0, cognitolessWalletFactories_1.createCognitolessWalletFactories)(platformCryptographyTools, component);
    const publicToolsFactories = (0, publicToolsFactories_1.createPublicToolsFactories)(platformCryptographyTools, component);
    return {
        ...cognitoFactories,
        ...cognitolessFactories,
        ...publicToolsFactories,
    };
};
exports.createV6WalletFactories = createV6WalletFactories;
//# sourceMappingURL=walletFactories.js.map