"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const tools_common_1 = require("@affinidi/tools-common");
const _helpers_1 = require("../_helpers");
let AffinidiVaultEncryptionService = class AffinidiVaultEncryptionService {
    constructor(keyManager) {
        this._keyManager = keyManager;
    }
    async encryptCredentials(credentials) {
        const encryptedCredentials = [];
        for (const originalCredential of credentials) {
            const { credential } = originalCredential;
            const credentialId = (0, _helpers_1.isW3cCredential)(credential) ? credential.id : credential.data.id;
            const idHash = await this._keyManager.computePersonalHash(credentialId);
            const typeHashes = [];
            if ((0, _helpers_1.isW3cCredential)(credential)) {
                for (const credentialType of credential.type) {
                    const typeHash = await this._keyManager.computePersonalHash(credentialType);
                    typeHashes.push(typeHash);
                }
            }
            const cyphertext = await this._keyManager.encryptByPublicKey(credential);
            encryptedCredentials.push({
                idHash,
                typeHashes,
                cyphertext,
                originalCredential,
            });
        }
        return encryptedCredentials;
    }
    async computeTypesHashes(types) {
        const hashedTypes = [];
        for (const subset of types) {
            const hashedSubset = [];
            for (const type of subset) {
                const hashedType = await this._keyManager.computePersonalHash(type);
                hashedSubset.push(hashedType);
            }
            hashedTypes.push(hashedSubset);
        }
        return hashedTypes;
    }
    async decryptCredentials(encryptedCredentials) {
        let credentials = [];
        const decryptCredentialsPromises = encryptedCredentials.map(async (encryptedCredential) => {
            try {
                const result = await this.decryptCredential(encryptedCredential);
                return result;
            }
            catch (error) {
                console.log('error decryptCredential', error);
            }
        });
        const data = await Promise.all(decryptCredentialsPromises);
        credentials = data.filter((credential) => !!credential);
        return credentials;
    }
    async decryptCredential(encryptedCredential) {
        const credential = await this._keyManager.decryptByPrivateKey(encryptedCredential.payload);
        return credential;
    }
    async computeHashedId(credentialId) {
        const hashedId = await this._keyManager.computePersonalHash(credentialId);
        return hashedId;
    }
};
AffinidiVaultEncryptionService = __decorate([
    (0, tools_common_1.profile)()
], AffinidiVaultEncryptionService);
exports.default = AffinidiVaultEncryptionService;
//# sourceMappingURL=AffinidiVaultEncryptionService.js.map