"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const internal_api_clients_1 = require("@affinidi/internal-api-clients");
const tools_common_1 = require("@affinidi/tools-common");
const _helpers_1 = require("../_helpers");
let AffinidiVaultStorageService = class AffinidiVaultStorageService {
    constructor(encryptionService, options) {
        this._encryptionService = encryptionService;
        this._vaultApiService = new internal_api_clients_1.AffinidiVaultApiService({
            vaultUrl: options.vaultUrl,
            accessApiKey: options.accessApiKey,
            sdkVersion: (0, _helpers_1.extractSDKVersion)(),
            didAuthAdapter: options.didAuthAdapter,
        });
    }
    async saveCredentials(credentials, storageRegion) {
        const responses = [];
        const encryptedCredentials = await this._encryptionService.encryptCredentials(credentials.map((credential) => ({ credential })));
        for (const credential of encryptedCredentials) {
            const { body } = await this._vaultApiService.storeCredential(storageRegion, credential.idHash, {
                credentialTypes: credential.typeHashes,
                payload: credential.cyphertext,
            });
            responses.push(body);
        }
        return responses;
    }
    async searchCredentials(storageRegion, types) {
        const hashedTypes = types ? await this._encryptionService.computeTypesHashes(types) : undefined;
        const { body } = await this._vaultApiService.searchCredentials(storageRegion, hashedTypes);
        const credentials = await this._encryptionService.decryptCredentials(body.credentials);
        return credentials;
    }
    async getCredentialById(credentialId, storageRegion) {
        const hashedId = await this._encryptionService.computeHashedId(credentialId);
        const { body } = await this._vaultApiService.getCredential(storageRegion, hashedId);
        const credential = await this._encryptionService.decryptCredential(body);
        return credential;
    }
    async deleteCredentialById(credentialId, storageRegion) {
        const hashedId = await this._encryptionService.computeHashedId(credentialId);
        await this._vaultApiService.deleteCredential(storageRegion, hashedId);
    }
    async deleteAllCredentials(storageRegion) {
        const { body } = await this._vaultApiService.searchCredentials(storageRegion);
        // this could have perfomance problems when need delete big amount of credentials
        // probably can use some kind of concurrency, but it is unknown what behavior will be at each platform
        for (const credential of body.credentials) {
            await this._vaultApiService.deleteCredential(storageRegion, credential.credentialId);
        }
    }
};
AffinidiVaultStorageService = __decorate([
    (0, tools_common_1.profile)()
], AffinidiVaultStorageService);
exports.default = AffinidiVaultStorageService;
//# sourceMappingURL=AffinidiVaultStorageService.js.map