"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@affinidi/common");
const internal_api_clients_1 = require("@affinidi/internal-api-clients");
const tools_common_1 = require("@affinidi/tools-common");
const async_retry_1 = __importDefault(require("async-retry"));
const _helpers_1 = require("../_helpers");
const createHash = require('create-hash');
const getDidData_1 = require("../shared/getDidData");
const SdkErrorFromCode_1 = __importDefault(require("../shared/SdkErrorFromCode"));
const sha256 = (data) => {
    return createHash('sha256').update(data).digest();
};
const hashFromString = (data) => {
    const buffer = sha256(Buffer.from(data));
    return buffer.toString('hex');
};
let KeyManagementService = class KeyManagementService {
    constructor(options) {
        this._keyStorageApiService = new internal_api_clients_1.KeyStorageApiService({
            keyStorageUrl: options.keyStorageUrl,
            accessApiKey: options.accessApiKey,
            sdkVersion: (0, _helpers_1.extractSDKVersion)(),
            tenantToken: options.tenantToken,
        });
    }
    async _pullEncryptedSeed(accessToken) {
        const { body: { encryptedSeed }, } = await this._keyStorageApiService.readMyKey({ accessToken });
        return encryptedSeed;
    }
    async _pullEncryptionKey(accessToken) {
        // TODO: must use key provider, its just a mock at this point
        const { payload } = common_1.JwtService.fromJWT(accessToken);
        const userId = payload.sub;
        const encryptionKey = hashFromString(userId);
        return encryptionKey;
    }
    async _storeEncryptedSeed(accessToken, seedHex, encryptionKey) {
        const encryptionKeyBuffer = Buffer.from(encryptionKey, 'hex');
        const encryptedSeed = await common_1.KeysService.encryptSeed(seedHex, encryptionKeyBuffer);
        await this._keyStorageApiService.storeMyKey(accessToken, { encryptedSeed });
    }
    async pullKeyAndSeed(accessToken) {
        const encryptionKey = await this._pullEncryptionKey(accessToken);
        const encryptedSeed = await this._pullEncryptedSeed(accessToken);
        return { encryptionKey, encryptedSeed };
    }
    async pullUserData(accessToken) {
        const { encryptionKey, encryptedSeed } = await this.pullKeyAndSeed(accessToken);
        return (0, getDidData_1.withDidData)({ encryptedSeed, password: encryptionKey });
    }
    async pullEncryptionKeyAndStoreEncryptedSeed(accessToken, seedHexWithMethod) {
        const encryptionKey = await this._pullEncryptionKey(accessToken);
        await this.storeEncryptedSeed(accessToken, seedHexWithMethod, encryptionKey);
    }
    async storeEncryptedSeed(accessToken, seedHexWithMethod, encryptionKey) {
        await (0, async_retry_1.default)(async (bail) => {
            const errorCodes = ['COR-1', 'WAL-2'];
            try {
                await this._storeEncryptedSeed(accessToken, seedHexWithMethod, encryptionKey);
            }
            catch (error) {
                if (errorCodes.indexOf(error.code) >= 0) {
                    // If it's a known error we can bail out of the retry and that error will be what's thrown
                    bail(error);
                    return;
                }
                else {
                    // Otherwise we wrap the error and throw that,
                    // this will trigger a retry until "retries" count is met
                    throw new SdkErrorFromCode_1.default('COR-18', { accessToken }, error);
                }
            }
        }, { retries: 3 });
    }
    /* To cover scenario when registration failed and private key is not saved:
     *    1. seed is generated before user is confirmed in Cognito
     *    2. encrypt seed with user's password
     *    3. confirm user in Cognito, if registration is successful
     *    4. get user's encryptionKey
     *    5. re-encrypt user's seed with encryptionKey
     */
    async reencryptSeed(accessToken, keyParams, backupUpdatedSeed) {
        const encryptionKey = await this._pullEncryptionKey(accessToken);
        const { fullSeedHex } = common_1.KeysService.decryptSeed(keyParams.encryptedSeed, keyParams.password);
        const encryptionKeyBuffer = common_1.KeysService.normalizePassword(encryptionKey);
        const updatedEncryptedSeed = await common_1.KeysService.encryptSeed(fullSeedHex, encryptionKeyBuffer);
        if (backupUpdatedSeed) {
            const { fullSeedHex } = common_1.KeysService.decryptSeed(updatedEncryptedSeed, encryptionKey);
            await this.storeEncryptedSeed(accessToken, fullSeedHex, encryptionKey);
        }
        return { encryptionKey, updatedEncryptedSeed };
    }
};
KeyManagementService = __decorate([
    (0, tools_common_1.profile)()
], KeyManagementService);
exports.default = KeyManagementService;
//# sourceMappingURL=KeyManagementService.js.map