"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@affinidi/common");
const internal_api_clients_1 = require("@affinidi/internal-api-clients");
const tools_common_1 = require("@affinidi/tools-common");
const _helpers_1 = require("../_helpers");
const SdkErrorFromCode_1 = __importDefault(require("../shared/SdkErrorFromCode"));
const AffinidiVaultStorageService_1 = __importDefault(require("./AffinidiVaultStorageService"));
const AffinidiVaultEncryptionService_1 = __importDefault(require("./AffinidiVaultEncryptionService"));
const createHash = require('create-hash');
const sha256 = (data) => {
    return createHash('sha256').update(data).digest();
};
let WalletStorageService = class WalletStorageService {
    constructor(keyManager, options) {
        this._storageRegion = options.storageRegion;
        const encryptionService = new AffinidiVaultEncryptionService_1.default(keyManager);
        this._affinidiVaultStorageService = new AffinidiVaultStorageService_1.default(encryptionService, {
            didAuthAdapter: options.didAuthAdapter,
            accessApiKey: options.accessApiKey,
            vaultUrl: options.affinidiVaultUrl,
        });
    }
    static hashFromString(data) {
        const buffer = sha256(Buffer.from(data));
        return buffer.toString('hex');
    }
    static async getCredentialOffer(accessToken, keyStorageUrl, options) {
        const { accessApiKey, env } = options;
        const service = new internal_api_clients_1.KeyStorageApiService({
            keyStorageUrl,
            accessApiKey,
            sdkVersion: (0, _helpers_1.extractSDKVersion)(),
        });
        const { body } = await service.getCredentialOffer({ accessToken, env });
        const { offerToken } = body;
        return offerToken;
    }
    static async getSignedCredentials(accessToken, credentialOfferResponseToken, options) {
        const keyStorageUrl = options.keyStorageUrl;
        const { env, issuerUrl, accessApiKey, apiKey } = options;
        const service = new internal_api_clients_1.KeyStorageApiService({ keyStorageUrl, accessApiKey, sdkVersion: (0, _helpers_1.extractSDKVersion)() });
        const { body } = await service.getSignedCredential(accessToken, {
            credentialOfferResponseToken,
            options: { env, issuerUrl, accessApiKey, apiKey },
        });
        const { signedCredentials } = body;
        return signedCredentials;
    }
    async _getCredentialsByTypes(storageRegion, types) {
        storageRegion = storageRegion || this._storageRegion;
        return this._affinidiVaultStorageService.searchCredentials(storageRegion, types);
    }
    async saveCredentials(credentials, storageRegion) {
        storageRegion = storageRegion || this._storageRegion;
        const responses = await this._affinidiVaultStorageService.saveCredentials(credentials, storageRegion);
        return responses;
    }
    async getAllCredentials(storageRegion) {
        storageRegion = storageRegion || this._storageRegion;
        return await this._affinidiVaultStorageService.searchCredentials(storageRegion);
    }
    async getCredentialsByShareToken(token, storageRegion) {
        storageRegion = storageRegion || this._storageRegion;
        if (!token) {
            return this.getAllCredentials();
        }
        const request = common_1.JwtService.fromJWT(token);
        const { payload: { interactionToken: { credentialRequirements }, }, } = request;
        if (!credentialRequirements) {
            return this.getAllCredentials();
        }
        const requirementTypes = credentialRequirements.map((credentialRequirement) => credentialRequirement.type);
        return this._getCredentialsByTypes(storageRegion, requirementTypes);
    }
    async getCredentialById(credentialId, storageRegion) {
        storageRegion = storageRegion || this._storageRegion;
        return await this._affinidiVaultStorageService.getCredentialById(credentialId, storageRegion);
    }
    async deleteCredentialById(credentialId, storageRegion) {
        storageRegion = storageRegion || this._storageRegion;
        await this._affinidiVaultStorageService.deleteCredentialById(credentialId, storageRegion);
    }
    async deleteAllCredentials(storageRegion) {
        storageRegion = storageRegion || this._storageRegion;
        try {
            await this._affinidiVaultStorageService.deleteAllCredentials(storageRegion);
        }
        catch (error) {
            throw new SdkErrorFromCode_1.default('COR-0', {}, error);
        }
    }
};
WalletStorageService = __decorate([
    (0, tools_common_1.profile)()
], WalletStorageService);
exports.default = WalletStorageService;
//# sourceMappingURL=WalletStorageService.js.map