"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.anchorDid = void 0;
const _defaultConfig_1 = require("../_defaultConfig");
const common_1 = require("@affinidi/common");
const computePreparedJoloParams = async ({ registry, keysService, nonce, additionalJoloParams }) => {
    if (!additionalJoloParams) {
        throw new Error('missing jolo params');
    }
    const { didDocument, seedHex } = additionalJoloParams;
    const did = didDocument.id;
    const signedDidDocument = await keysService.signDidDocument(didDocument);
    const { body: bodyDidDocument } = await registry.putDocumentInIpfs({ document: signedDidDocument });
    const didDocumentAddress = bodyDidDocument.hash;
    const { body: { digestHex }, } = await registry.createAnchorTransaction({ nonce, did, didDocumentAddress });
    const transactionSignatureJson = digestHex ? await keysService.createTransactionSignature(digestHex, seedHex) : '';
    const transactionPublicKey = common_1.KeysService.getAnchorTransactionPublicKey(seedHex, _defaultConfig_1.JOLO_DID_METHOD);
    const ethereumPublicKeyHex = transactionPublicKey.toString('hex');
    return { did, didDocumentAddress, ethereumPublicKeyHex, transactionSignatureJson, nonce };
};
const computePreparedElemParams = async ({ did }) => {
    return { did, didDocumentAddress: '', ethereumPublicKeyHex: '', transactionSignatureJson: '' };
};
const computePreparedPolygonParams = async ({ did, keysService, didMethod, registry }) => {
    const didService = new common_1.PolygonDidDocumentService(new common_1.LocalKeyVault(keysService), {
        isTestnet: didMethod === 'polygon:testnet',
    });
    const publicKeyBase58 = didService.getMyPubKeyBase58();
    const { body: { digestHex }, } = await registry.createAnchorTransaction({ did, publicKeyBase58 });
    const transactionSignatureJson = await keysService.createTransactionSignature(digestHex);
    return { did, publicKeyBase58, transactionSignatureJson };
};
const computePreparedAnchoringParams = async (params) => {
    const { didMethod } = params;
    switch (didMethod) {
        case _defaultConfig_1.JOLO_DID_METHOD:
            return computePreparedJoloParams(params);
        case _defaultConfig_1.ELEM_DID_METHOD:
        case _defaultConfig_1.ELEM_ANCHORED_DID_METHOD:
            return computePreparedElemParams(params);
        case _defaultConfig_1.POLYGON_DID_METHOD:
        case _defaultConfig_1.POLYGON_TESTNET_DID_METHOD:
            return computePreparedPolygonParams(params);
        default:
            throw new Error(`did method: "${didMethod}" is not supported`);
    }
};
const anchorDid = async (params) => {
    const { registry, anchoredDidElem, nonce } = params;
    const preparedParams = await computePreparedAnchoringParams(params);
    const response = await registry.anchorDid({
        ...preparedParams,
        nonce,
        anchoredDidElem,
        origin: params.origin,
    });
    return { did: response.body.did };
};
exports.anchorDid = anchorDid;
//# sourceMappingURL=anchoringHandler.js.map