"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const _defaultConfig_1 = require("../_defaultConfig");
const common_1 = require("@affinidi/common");
const anchoringHandler_1 = require("./anchoringHandler");
const seedTools_1 = require("@affinidi/common/dist/shared/seedTools");
const register = async (registry, didMethod, platformCryptographyTools, password, keyOptions, origin, skipAnchoringForElemMethod, webDomain) => {
    const isAnchoredSeed = didMethod === _defaultConfig_1.ELEM_ANCHORED_DID_METHOD;
    let seedMetadata;
    // TODO: figure out to use proper SDK error (not regular Error)
    if (didMethod && didMethod === 'web') {
        if (!webDomain) {
            throw Error('webDomain option should be provided to register did:web');
        }
        seedMetadata = { webDomain };
    }
    const seedWithMethod = await (0, common_1.generateFullSeed)(platformCryptographyTools, didMethod, keyOptions, seedMetadata);
    const { keysService, encryptedSeed } = await common_1.KeysService.fromSeedAndPassword(seedWithMethod, password);
    const didDocumentService = common_1.DidDocumentService.createDidDocumentService(keysService);
    const { did: builtDid, didDocument, keyId } = await didDocumentService.buildDidDocumentForRegister();
    let anchoredInBlockchainDid;
    if (didMethod !== _defaultConfig_1.KEY_DID_METHOD &&
        didMethod !== _defaultConfig_1.WEB_DID_METHOD &&
        (didMethod != _defaultConfig_1.ELEM_DID_METHOD || !skipAnchoringForElemMethod)) {
        const response = await (0, anchoringHandler_1.anchorDid)({
            registry,
            keysService,
            didMethod,
            did: builtDid,
            anchoredDidElem: isAnchoredSeed,
            nonce: 0,
            additionalJoloParams: {
                didDocument,
                seedHex: (0, seedTools_1.parseDecryptedSeed)(seedWithMethod).seed.toString('hex'),
            },
            origin,
        });
        anchoredInBlockchainDid = response.did;
    }
    if (isAnchoredSeed) {
        const anchoredSeed = (0, common_1.extendSeedWithDid)(seedWithMethod, anchoredInBlockchainDid);
        const { encryptedSeed: anchoredEncryptedSeed, keysService } = await common_1.KeysService.fromSeedAndPassword(anchoredSeed, password);
        const anchoredDocumentService = common_1.DidDocumentService.createDidDocumentService(keysService);
        const didDocumentKeyId = anchoredDocumentService.getKeyId();
        return { did: anchoredInBlockchainDid, encryptedSeed: anchoredEncryptedSeed, didDocumentKeyId };
    }
    const didDocumentKeyId = keyId;
    return { did: didDocumentService.getMyDid(), didDocumentKeyId, encryptedSeed, didDocument };
};
exports.register = register;
//# sourceMappingURL=registeringHandler.js.map