"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOptionsFromEnvironment = exports.getAccessApiKeyFromOptions = void 0;
const common_1 = require("@affinidi/common");
const url_resolver_1 = require("@affinidi/url-resolver");
const _defaultConfig_1 = require("../_defaultConfig");
const getAccessApiKeyFromOptions = ({ accessApiKey, apiKey }) => {
    if (!accessApiKey && !apiKey) {
        throw new Error('Neither accessApiKey nor apiKey is contained in options');
    }
    if (apiKey) {
        const apiKeyBuffer = common_1.KeysService.sha256(Buffer.from(apiKey));
        return apiKeyBuffer.toString('hex');
    }
    return accessApiKey;
};
exports.getAccessApiKeyFromOptions = getAccessApiKeyFromOptions;
function getBasicOptionsFromEnvironment(options) {
    const env = options.env;
    const urls = {
        issuerUrl: (0, url_resolver_1.resolveUrl)(url_resolver_1.Service.ISSUER, env, options.issuerUrl),
        registryUrl: (0, url_resolver_1.resolveUrl)(url_resolver_1.Service.REGISTRY, env, options.registryUrl),
        verifierUrl: (0, url_resolver_1.resolveUrl)(url_resolver_1.Service.VERIFIER, env, options.verifierUrl),
        affinidiVaultUrl: (0, url_resolver_1.resolveUrl)(url_resolver_1.Service.VAULT, env, options.affinidiVaultUrl),
        keyStorageUrl: (0, url_resolver_1.resolveUrl)(url_resolver_1.Service.KEY_STORAGE, env, options.keyStorageUrl),
        revocationUrl: (0, url_resolver_1.resolveUrl)(url_resolver_1.Service.REVOCATION, env, options.revocationUrl),
        metricsUrl: (0, url_resolver_1.resolveUrl)(url_resolver_1.Service.METRICS, env),
        migrationUrl: (0, url_resolver_1.resolveUrl)(url_resolver_1.Service.VAULT_MIGRATION, env),
    };
    switch (env) {
        /* istanbul ignore next */
        case 'dev':
            return {
                env: env,
                clientId: options.clientId || _defaultConfig_1.DEV_COGNITO_CLIENT_ID,
                userPoolId: options.userPoolId || _defaultConfig_1.DEV_COGNITO_USER_POOL_ID,
                phoneIssuerBasePath: options.phoneIssuerBasePath || _defaultConfig_1.DEV_PHONE_ISSUER_BASE_PATH,
                emailIssuerBasePath: options.emailIssuerBasePath || _defaultConfig_1.DEV_EMAIL_ISSUER_BASE_PATH,
                ...urls,
            };
        /* istanbul ignore next */
        case 'prod':
            return {
                env: env,
                clientId: options.clientId || _defaultConfig_1.PROD_COGNITO_CLIENT_ID,
                userPoolId: options.userPoolId || _defaultConfig_1.PROD_COGNITO_USER_POOL_ID,
                phoneIssuerBasePath: options.phoneIssuerBasePath || _defaultConfig_1.PROD_PHONE_ISSUER_BASE_PATH,
                emailIssuerBasePath: options.emailIssuerBasePath || _defaultConfig_1.PROD_EMAIL_ISSUER_BASE_PATH,
                ...urls,
            };
        case 'staging':
            return {
                env: env,
                clientId: options.clientId || _defaultConfig_1.STAGING_COGNITO_CLIENT_ID,
                userPoolId: options.userPoolId || _defaultConfig_1.STAGING_COGNITO_USER_POOL_ID,
                phoneIssuerBasePath: options.phoneIssuerBasePath || _defaultConfig_1.STAGING_PHONE_ISSUER_BASE_PATH,
                emailIssuerBasePath: options.emailIssuerBasePath || _defaultConfig_1.STAGING_EMAIL_ISSUER_BASE_PATH,
                ...urls,
            };
    }
}
const splitOptions = (options) => {
    const { accessApiKey, apiKey, env, issuerUrl, registryUrl, verifierUrl, affinidiVaultUrl, keyStorageUrl, phoneIssuerBasePath, emailIssuerBasePath, revocationUrl, storageRegion, clientId, userPoolId, region, origin, ...otherOptions } = options;
    return {
        region,
        accessApiKeyOptions: {
            accessApiKey,
            apiKey,
        },
        environmentOptions: {
            env,
            issuerUrl,
            registryUrl,
            verifierUrl,
            affinidiVaultUrl,
            keyStorageUrl,
            phoneIssuerBasePath,
            emailIssuerBasePath,
            revocationUrl,
            clientId,
            userPoolId,
        },
        storageRegion,
        otherOptions,
        origin,
    };
};
const getOptionsFromEnvironment = (options) => {
    const { region, accessApiKeyOptions, environmentOptions, storageRegion, otherOptions, origin } = splitOptions(options);
    return {
        region: region || _defaultConfig_1.DEFAULT_COGNITO_REGION,
        basicOptions: getBasicOptionsFromEnvironment(environmentOptions),
        accessApiKey: (0, exports.getAccessApiKeyFromOptions)(accessApiKeyOptions),
        storageRegion,
        otherOptions,
        origin,
    };
};
exports.getOptionsFromEnvironment = getOptionsFromEnvironment;
//# sourceMappingURL=getOptionsFromEnvironment.js.map