"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SessionStorageService = void 0;
const SdkErrorFromCode_1 = __importDefault(require("./SdkErrorFromCode"));
let tempSessionStorage = {};
// NOTE: isBrowser is defined in this way because in EXPO
//       typeof document === object
//       typeof window === object
const isBrowser = typeof navigator !== 'undefined' && navigator.product !== 'ReactNative';
const isTestEnvironment = process.env.NODE_ENV === 'test';
class SessionStorageService {
    constructor(sessionStorageKey) {
        this._sessionStorageKey = sessionStorageKey;
    }
    readUserTokens() {
        let sessionStorageObject;
        if (isBrowser) {
            sessionStorageObject = window.sessionStorage.getItem(this._sessionStorageKey);
        }
        if (isTestEnvironment) {
            sessionStorageObject = tempSessionStorage[this._sessionStorageKey];
        }
        if (!sessionStorageObject) {
            throw new SdkErrorFromCode_1.default('COR-9');
        }
        return JSON.parse(sessionStorageObject);
    }
    saveUserTokens(cognitoUserTokens) {
        const sessionStorageObject = JSON.stringify(cognitoUserTokens);
        if (isBrowser) {
            window.sessionStorage.setItem(this._sessionStorageKey, sessionStorageObject);
        }
        if (isTestEnvironment) {
            tempSessionStorage[this._sessionStorageKey] = sessionStorageObject;
        }
    }
    clearUserTokens() {
        if (isBrowser) {
            window.sessionStorage.removeItem(this._sessionStorageKey);
        }
        if (isTestEnvironment) {
            tempSessionStorage = {};
        }
    }
}
exports.SessionStorageService = SessionStorageService;
//# sourceMappingURL=sessionStorageHandler.js.map